/*
 * Meva
 * Version:  0.1
 * Copyright (C) 1997  Dedieu Sebastien
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef MEVA_NATIVE_DEFINITIONS_H
#define MEVA_NATIVE_DEFINITIONS_H


#ifdef JNI


#include "jni.h"


/* define MEVA_NATIVE(returned_type, method_name, jniSignature, jriSignature) returned_type JNICALL Java_ ## method_name ## jniSignature */
#define MEVA_NATIVE(returned_type, method_name) returned_type JNICALL Java_ ## method_name


typedef JNIEnv MEVA_ENV;


#define MEVA_GET_BOOLEAN_ARRAY(env, array, copy) (*env)->GetBooleanArrayElements(env, array, 0)
#define MEVA_GET_BYTE_ARRAY(env, array, copy) (*env)->GetByteArrayElements(env, array, 0)
#define MEVA_GET_SHORT_ARRAY(env, array, copy) (*env)->GetShortArrayElements(env, array, 0)
#define MEVA_GET_INT_ARRAY(env, array, copy) (*env)->GetIntArrayElements(env, array, 0)
#define MEVA_GET_LONG_ARRAY(env, array, copy) (*env)->GetLongArrayElements(env, array, 0)
#define MEVA_GET_FLOAT_ARRAY(env, array, copy) (*env)->GetFloatArrayElements(env, array, 0)
#define MEVA_GET_DOUBLE_ARRAY(env, array, copy) (*env)->GetDoubleArrayElements(env, array, 0)


#define MEVA_RELEASE_BOOLEAN_ARRAY(env, array, pointer, mode) (*env)->ReleaseBooleanArrayElements(env, array, pointer, mode)
#define MEVA_RELEASE_BYTE_ARRAY(env, array, pointer, mode) (*env)->ReleaseByteArrayElements(env, array, pointer, mode)
#define MEVA_RELEASE_SHORT_ARRAY(env, array, pointer, mode) (*env)->ReleaseShortArrayElements(env, array, pointer, mode)
#define MEVA_RELEASE_INT_ARRAY(env, array, pointer, mode) (*env)->ReleaseIntArrayElements(env, array, pointer, mode)
#define MEVA_RELEASE_LONG_ARRAY(env, array, pointer, mode) (*env)->ReleaseLongArrayElements(env, array, pointer, mode)
#define MEVA_RELEASE_FLOAT_ARRAY(env, array, pointer, mode) (*env)->ReleaseFloatArrayElements(env, array, pointer, mode)
#define MEVA_RELEASE_DOUBLE_ARRAY(env, array, pointer, mode) (*env)->ReleaseDoubleArrayElements(env, array, pointer, mode)


#define MEVA_GET_STRING_UTF(env, string, copy) ((*env)->GetStringUTFChars(env, string, copy))
#define MEVA_RELEASE_STRING_UTF(env, string, pointer)  (*env)->ReleaseStringUTFChars(env, string, pointer)


#endif


#ifdef JRI


#include "jri.h"


/* define MEVA_NATIVE(returned_type, method_name, jniSignature, jriSignature) JRI_PUBLIC_API(returned_type) native_ ## method_name ## jriSignature */
#define MEVA_NATIVE(returned_type, method_name) JRI_PUBLIC_API(returned_type) native_ ## method_name


typedef JRIEnv MEVA_ENV;
typedef struct java_lang_Class *jclass;
typedef jbool jboolean;


#define MEVA_GET_BOOLEAN_ARRAY(env, array, copy) JRI_GetByteArrayElements(env, array) /* Boolean -> Byte */
#define MEVA_GET_BYTE_ARRAY(env, array, copy) JRI_GetByteArrayElements(env, array)
#define MEVA_GET_SHORT_ARRAY(env, array, copy) JRI_GetShortArrayElements(env, array)
#define MEVA_GET_INT_ARRAY(env, array, copy) JRI_GetIntArrayElements(env, array)
#define MEVA_GET_LONG_ARRAY(env, array, copy) JRI_GetLongArrayElements(env, array)
#define MEVA_GET_FLOAT_ARRAY(env, array, copy) JRI_GetFloatArrayElements(env, array)
#define MEVA_GET_DOUBLE_ARRAY(env, array, copy) JRI_GetDoubleArrayElements(env, array)


#define MEVA_RELEASE_BOOLEAN_ARRAY(env, array, pointer, mode) ((void)0)
#define MEVA_RELEASE_BYTE_ARRAY(env, array, pointer, mode) ((void)0)
#define MEVA_RELEASE_SHORT_ARRAY(env, array, pointer, mode) ((void)0)
#define MEVA_RELEASE_INT_ARRAY(env, array, pointer, mode) ((void)0)
#define MEVA_RELEASE_LONG_ARRAY(env, array, pointer, mode) ((void)0)
#define MEVA_RELEASE_FLOAT_ARRAY(env, array, pointer, mode) ((void)0)
#define MEVA_RELEASE_DOUBLE_ARRAY(env, array, pointer, mode) ((void)0)


typedef struct java_lang_String *jstring;


#define MEVA_GET_STRING_UTF(env, string, copy) JRI_GetStringUTFChars(env, string)
#define MEVA_RELEASE_STRING_UTF(env, string, pointer)  ((void)0)


#endif


#endif






