%{


/*
 * Meva
 * Version:  0.1
 * Copyright (C) 1997  Dedieu Sebastien
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>


typedef char *String;
#define YYSTYPE String


char *module;


enum 
{
  JAVA_MODE=1,
  C_MODE=2
} mode=0;


char *method, *returnedType; /* , *jniSignature, *jriSignature; */
void (*convertMethod)();


struct
{
  char *type;
  char *name;
  int isAPointer;
} argTable[128];


int nbArgs=0;


int yyerror(char *m);
int main(int argc, char *argv[]);
void convertMethodToJava();
void convertMethodToC();
char *convertType(char *type);
void argTableAdd(char *type, char *name, int isAPointer);
void argTableClear();


%}



%token STRING


%%


input : { }
      | input function { }
      ;


param : STRING STRING     { argTableAdd($1, $2, 0); }
      | STRING '*' STRING { argTableAdd($1, $3, 1); }
      ;


nextParams : { }
           | ',' param nextParams { }
           ;


paramList : '(' ')' { }
          | '(' param nextParams ')' { }
          ;


function : STRING STRING paramList ';' {
  int voidIndex=0;
  int hasVoidPointerArg=0;
  int argCount;
  method=$2;
  returnedType=convertType($1);

  for (argCount=0; argCount<nbArgs; argCount++)
  {
    if (strcmp(argTable[argCount].type, "GLvoid")==0)
    {
      assert(argTable[argCount].isAPointer);
      assert(!hasVoidPointerArg);
      hasVoidPointerArg=1;
      voidIndex=argCount;
    }    
  }

  if (!hasVoidPointerArg)
  {
/*    jniSignature="";
    jriSignature=""; */
    convertMethod();
  }
  else
  {
/*
    char jniTmp[256];
    int jniSignatureLength, argCount;

    jniSignature=jniTmp;
    strcpy(jniSignature, "__");
    for (argCount=0; argCount<nbArgs; argCount++)
      strcat(jniSignature, "I");
    strcat(jniSignature, "_3B");
    jniSignatureLength=strlen(jniSignature);

    jniSignature[jniSignatureLength-1]='B';
    jriSignature="";
    argTable[voidIndex].type="GLbyte";
    convertMethod();

    jniSignature[jniSignatureLength-1]='S';
    jriSignature="_1";
    argTable[voidIndex].type="GLshort";
    convertMethod();

    jniSignature[jniSignatureLength-1]='I';
    jriSignature="_2";
    argTable[voidIndex].type="GLint";
    convertMethod();

    jniSignature[jniSignatureLength-1]='F';
    jriSignature="_3";
    argTable[voidIndex].type="GLfloat";
    convertMethod();

    jniSignature[jniSignatureLength-1]='D';
    jriSignature="_4";
    argTable[voidIndex].type="GLdouble";
    convertMethod();

    jniSignature[jniSignatureLength-1]='Z';
    jriSignature="_5";
    argTable[voidIndex].type="GLboolean";
    convertMethod();
*/
  }

  argTableClear();
}
         ;


%%


#include "genFlex.c"


int yyerror(char *m)
{
  fprintf(stderr, "%s\n", m);
}


int main(int argc, char *argv[])
{
  char tmp[256];
  int outFd;

  assert(argc==3);
  module=argv[1];
  if (strcmp(argv[2], "java")==0)
  {
    mode=JAVA_MODE;
    convertMethod=convertMethodToJava;
  }
  else if (strcmp(argv[2], "c")==0)
  {
    mode=C_MODE;
    convertMethod=convertMethodToC;
  }
  else
    assert(0);

  if (mode==JAVA_MODE)
    sprintf(tmp, "%c%s.java", toupper(*module), module+1);
  else
    sprintf(tmp, "%c%s.c", toupper(*module), module+1);
  close(1);
  outFd=open(tmp, O_CREAT|O_TRUNC|O_WRONLY, 0666);
  assert(outFd!=-1);

  sprintf(tmp, "%sApi.c", module);
  yyin=fopen(tmp, "r");
  assert(yyin!=NULL);

  if (mode==JAVA_MODE)
  {
    sprintf(tmp, "%c%s", toupper(*module), module+1);
    printf("class %s\n", tmp);
    printf("{\n");
    sprintf(tmp, "cat %sConst.java", module);
    fflush(stdout);
    system(tmp);
    fflush(stdout);
  }
  else
  {
    printf("#include <stdlib.h>\n");
    printf("#include <stdio.h>\n");
    printf("#include <GL/%s.h>\n", module);
    printf("#include \"MevaNativeDefinitions.h\"\n");
    printf("#include \"%c%s.h\"\n", toupper(*module), module+1);
    printf("\n");
    printf("\n");
  }

  if (mode==JAVA_MODE)
    printf("\n");

  yyparse();

  if (mode==JAVA_MODE)
    printf("}\n");

  fflush(stdout);

  return 0;
}


void convertMethodToJava()
{
  int argCount;

  printf("  public static native %s %s(", returnedType, method);
  
  for (argCount=0; argCount<nbArgs; argCount++)
  {
    if (argTable[argCount].isAPointer)
      printf("%s %s[]", convertType(argTable[argCount].type), argTable[argCount].name);
    else
      printf("%s %s", convertType(argTable[argCount].type), argTable[argCount].name);

    if (argCount!=nbArgs-1)
      printf(", ");
  }

  printf(");\n");
}


void convertMethodToC()
{
  int argCount;
  int lineFlag=0;

/*
  if (strcmp(returnedType, "void")==0)
    printf("MEVA_NATIVE(%s, %c%s_%s, %s, %s)(MEVA_ENV *_env, jclass _class", returnedType, toupper(*module), module+1, method, jniSignature, jriSignature);
  else
    printf("MEVA_NATIVE(j%s, %c%s_%s, %s, %s)(MEVA_ENV *_env, jclass _class", returnedType, toupper(*module), module+1, method, jniSignature, jriSignature);
*/

  if (strcmp(returnedType, "void")==0)
    printf("MEVA_NATIVE(%s, %c%s_%s)(MEVA_ENV *_env, jclass _class", returnedType, toupper(*module), module+1, method);
  else
    printf("MEVA_NATIVE(j%s, %c%s_%s)(MEVA_ENV *_env, jclass _class", returnedType, toupper(*module), module+1, method);
  
  if (nbArgs!=0)
    printf(", ");

  for (argCount=0; argCount<nbArgs; argCount++)
  {
    if (argTable[argCount].isAPointer)
      printf("j%sArray %s", convertType(argTable[argCount].type), argTable[argCount].name);
    else
      printf("j%s %s", convertType(argTable[argCount].type), argTable[argCount].name);

    if (argCount!=nbArgs-1)
      printf(", ");
  }

  printf(")\n"); 
  printf("{\n");

  if (strcmp(returnedType, "void")!=0)
  {
    printf("  j%s _returned_value;\n", returnedType);
    lineFlag=1;
  }

  for (argCount=0; argCount<nbArgs; argCount++)
  {
    if (argTable[argCount].isAPointer)
    {
      char *tmp;
      char *type=strdup(convertType(argTable[argCount].type));
      for (tmp=type; (*tmp)!='\0'; tmp++)
        *tmp=toupper(*tmp);

      printf("  j%s *_%s=MEVA_GET_%s_ARRAY(_env, %s, 0);\n", convertType(argTable[argCount].type), argTable[argCount].name, type, argTable[argCount].name);

      lineFlag=1;
    }
  }

  if (lineFlag==1)
    printf("\n");

  printf("  ");

  if (strcmp(returnedType, "void")!=0)
    printf("_returned_value=");

  printf("%s(", method);

  for (argCount=0; argCount<nbArgs; argCount++)
  {
    if (argTable[argCount].isAPointer)
      printf("(%s *)_%s", argTable[argCount].type, argTable[argCount].name);
    else
      printf("(%s)%s", argTable[argCount].type, argTable[argCount].name);

    if (argCount!=nbArgs-1)
      printf(", ");
  }

  printf(");\n");

  lineFlag=0;

  for (argCount=0; argCount<nbArgs; argCount++)
  {
    if (argTable[argCount].isAPointer)
    {
      char *tmp;
      char *type=strdup(convertType(argTable[argCount].type));
      for (tmp=type; (*tmp)!='\0'; tmp++)
        *tmp=toupper(*tmp);

      if (lineFlag==0)
      {  
         printf("\n"); 
         lineFlag=1;
      }

      printf("  MEVA_RELEASE_%s_ARRAY(_env, %s, _%s, 0);\n", type, argTable[argCount].name, argTable[argCount].name); 
    }
  }

  if (strcmp(returnedType, "void")!=0)
  {
    printf("\n");
    printf("  return _returned_value;\n");
  }

  printf("}\n");
  printf("\n");
  printf("\n");
}


char *convertType(char *type)
{
  if (strcmp(type, "GLenum")==0)
    return "int";

  if (strcmp(type, "GLubyte")==0)
    return "byte";

  if (strcmp(type, "GLushort")==0)
    return "short";

  if (strcmp(type, "GLuint")==0)
    return "int";

  if (strcmp(type, "GLbyte")==0)
    return "byte";

  if (strcmp(type, "GLshort")==0)
    return "short";

  if (strcmp(type, "GLint")==0)
    return "int";

  if (strcmp(type, "GLfloat")==0)
    return "float";

  if (strcmp(type, "GLdouble")==0)
    return "double";

  if (strcmp(type, "GLboolean")==0)
    return "boolean";

  if (strcmp(type, "GLvoid")==0)
    return "void";

  if (strcmp(type, "GLfloat")==0)
    return "float";

  if (strcmp(type, "GLbitfield")==0)
    return "int";

  if (strcmp(type, "GLsizei")==0)
    return "int";

  if (strcmp(type, "GLclampf")==0)
    return "float";

  if (strcmp(type, "GLclampd")==0)
    return "double";

  assert(0); 

  return NULL;
}


void argTableAdd(char *type, char *name, int isAPointer)
{
  assert(nbArgs<128);

  argTable[nbArgs].type=type;
  argTable[nbArgs].name=name;
  argTable[nbArgs].isAPointer=isAPointer;
  nbArgs++;
}


void argTableClear()
{
  nbArgs=0;
}
